@@ -24,7 +24,7 @@
 #include <string.h>
 #include <time.h>
 
-#include <gtk/gtk.h>
+#include <glib.h>
 #include <libgnomevfs/gnome-vfs-types.h>
 #include <libgnomevfs/gnome-vfs-mime.h>
 
@@ -33,13 +33,9 @@
 #include "fr-command.h"
 #include "fr-command-tar.h"
 
-
 static void fr_command_tar_class_init  (FRCommandTarClass *class);
-
-static void fr_command_tar_init        (FRCommand *afile);
-
-static void fr_command_tar_destroy     (GtkObject *object);
-
+static void fr_command_tar_init        (FRCommand         *afile);
+static void fr_command_tar_finalize    (GObject           *object);
 
 /* Parent Class */
 
@@ -49,10 +45,11 @@ static FRCommandClass *parent_class = NULL;
 /* -- list -- */
 
 static time_t
-mktime_from_string (char *date_s, char *time_s)
+mktime_from_string (char *date_s, 
+		    char *time_s)
 {
-	struct tm tm = {0, };
-	char **fields;
+	struct tm   tm = {0, };
+	char      **fields;
 
 	/* date */
 
@@ -94,11 +91,12 @@ eat_spaces (char *line)
 
 
 static char **
-split_line (char *line, int n_fields)
+split_line (char *line, 
+	    int   n_fields)
 {
 	char **fields;
-	char *scan, *field_end;
-	int i;
+	char  *scan, *field_end;
+	int    i;
 
 	fields = g_new0 (char *, n_fields + 1);
 	fields[n_fields + 1] = NULL;
@@ -117,9 +115,9 @@ split_line (char *line, int n_fields)
 static char *
 get_last_field (char *line)
 {
-	int i;
+	int   i;
 	char *field;
-	int n = 6;
+	int   n = 6;
 
 	n--;
 	field = eat_spaces (line);
@@ -133,12 +131,13 @@ get_last_field (char *line)
 
 
 static void
-process_line (char *line, gpointer data)
+process_line (char     *line, 
+	      gpointer  data)
 {
-	FileData *fdata;
-	FRCommand *comm = FR_COMMAND (data);
-	char **fields;
-	char *name_field;
+	FileData   *fdata;
+	FRCommand  *comm = FR_COMMAND (data);
+	char      **fields;
+	char       *name_field;
 
 	g_return_if_fail (line != NULL);
 
@@ -219,24 +218,30 @@ add_compress_arg (FRCommand *comm)
 static void
 fr_command_tar_list (FRCommand *comm)
 {
+	fr_process_set_out_line_func (FR_COMMAND (comm)->process, 
+				      process_line,
+				      comm);
+
 	fr_process_clear (comm->process);
 	fr_process_begin_command (comm->process, "tar");
 	fr_process_add_arg (comm->process, "-tvf");
 	fr_process_add_arg (comm->process, comm->e_filename);
 	add_compress_arg (comm);
 	fr_process_end_command (comm->process);
-	fr_process_start (comm->process, TRUE);
+	fr_process_start (comm->process);
 }
 
 
 static void
-fr_command_tar_add (FRCommand *comm,
-		    GList *file_list,
-		    gchar *base_dir,
-		    gboolean update)
+fr_command_tar_add (FRCommand     *comm,
+		    GList         *file_list,
+		    const char    *base_dir,
+		    gboolean       update,
+		    const char    *password,
+		    FRCompression  compression)
 {
 	FRCommandTar *c_tar = FR_COMMAND_TAR (comm);
-	GList *scan;
+	GList        *scan;
 
 	/* Add files. */
 
@@ -263,10 +268,10 @@ fr_command_tar_add (FRCommand *comm,
 
 static void
 fr_command_tar_delete (FRCommand *comm,
-		       GList *file_list)
+		       GList     *file_list)
 {
 	FRCommandTar *c_tar = FR_COMMAND_TAR (comm);
-	GList *scan;
+	GList        *scan;
 
 	/* Delete files. */
 
@@ -282,12 +287,13 @@ fr_command_tar_delete (FRCommand *comm,
 
 
 static void
-fr_command_tar_extract (FRCommand *comm,
-			GList *file_list,
-			char *dest_dir,
-			gboolean overwrite,
-			gboolean skip_older,
-			gboolean junk_paths)
+fr_command_tar_extract (FRCommand  *comm,
+			GList      *file_list,
+			const char *dest_dir,
+			gboolean    overwrite,
+			gboolean    skip_older,
+			gboolean    junk_paths,
+			const char *password)
 {
 	GList *scan;
 
@@ -312,10 +318,11 @@ fr_command_tar_extract (FRCommand *comm,
 
 
 static void
-fr_command_tar_recompress (FRCommand *comm)
+fr_command_tar_recompress (FRCommand     *comm,
+			   FRCompression  compression)
 {
 	FRCommandTar *c_tar = FR_COMMAND_TAR (comm);
-	gchar *new_name = NULL;
+	gchar        *new_name = NULL;
 
 	switch (c_tar->compress_prog) {
 	case FR_COMPRESS_PROGRAM_NONE:
@@ -324,6 +331,16 @@ fr_command_tar_recompress (FRCommand *comm)
 	case FR_COMPRESS_PROGRAM_GZIP:
 		fr_process_begin_command (comm->process, "gzip");
 		fr_process_set_sticky (comm->process, TRUE);
+		switch (compression) {
+		case FR_COMPRESSION_VERY_FAST:
+			fr_process_add_arg (comm->process, "-1"); break;
+		case FR_COMPRESSION_FAST:
+			fr_process_add_arg (comm->process, "-3"); break;
+		case FR_COMPRESSION_NORMAL:
+			fr_process_add_arg (comm->process, "-6"); break;
+		case FR_COMPRESSION_MAXIMUM:
+			fr_process_add_arg (comm->process, "-9"); break;
+		}
 		fr_process_add_arg (comm->process, "-f");
 		fr_process_add_arg (comm->process, c_tar->uncomp_filename);
 		fr_process_end_command (comm->process);
@@ -334,6 +351,16 @@ fr_command_tar_recompress (FRCommand *comm)
 	case FR_COMPRESS_PROGRAM_BZIP:
 		fr_process_begin_command (comm->process, "bzip");
 		fr_process_set_sticky (comm->process, TRUE);
+		switch (compression) {
+		case FR_COMPRESSION_VERY_FAST:
+			fr_process_add_arg (comm->process, "-1"); break;
+		case FR_COMPRESSION_FAST:
+			fr_process_add_arg (comm->process, "-3"); break;
+		case FR_COMPRESSION_NORMAL:
+			fr_process_add_arg (comm->process, "-6"); break;
+		case FR_COMPRESSION_MAXIMUM:
+			fr_process_add_arg (comm->process, "-9"); break;
+		}
 		fr_process_add_arg (comm->process, "-f");
 		fr_process_add_arg (comm->process, c_tar->uncomp_filename);
 		fr_process_end_command (comm->process);
@@ -344,6 +371,16 @@ fr_command_tar_recompress (FRCommand *comm)
 	case FR_COMPRESS_PROGRAM_BZIP2:
 		fr_process_begin_command (comm->process, "bzip2");
 		fr_process_set_sticky (comm->process, TRUE);
+		switch (compression) {
+		case FR_COMPRESSION_VERY_FAST:
+			fr_process_add_arg (comm->process, "-1"); break;
+		case FR_COMPRESSION_FAST:
+			fr_process_add_arg (comm->process, "-3"); break;
+		case FR_COMPRESSION_NORMAL:
+			fr_process_add_arg (comm->process, "-6"); break;
+		case FR_COMPRESSION_MAXIMUM:
+			fr_process_add_arg (comm->process, "-9"); break;
+		}
 		fr_process_add_arg (comm->process, "-f");
 		fr_process_add_arg (comm->process, c_tar->uncomp_filename);
 		fr_process_end_command (comm->process);
@@ -364,7 +401,18 @@ fr_command_tar_recompress (FRCommand *comm)
 	case FR_COMPRESS_PROGRAM_LZOP:
 		fr_process_begin_command (comm->process, "lzop");
 		fr_process_set_sticky (comm->process, TRUE);
+		switch (compression) {
+		case FR_COMPRESSION_VERY_FAST:
+			fr_process_add_arg (comm->process, "-1"); break;
+		case FR_COMPRESSION_FAST:
+			fr_process_add_arg (comm->process, "-3"); break;
+		case FR_COMPRESSION_NORMAL:
+			fr_process_add_arg (comm->process, "-6"); break;
+		case FR_COMPRESSION_MAXIMUM:
+			fr_process_add_arg (comm->process, "-9"); break;
+		}
 		fr_process_add_arg (comm->process, "-fU");
+		fr_process_add_arg (comm->process, "--no-stdin");
 		fr_process_add_arg (comm->process, c_tar->uncomp_filename);
 		fr_process_end_command (comm->process);
 
@@ -393,8 +441,8 @@ static void
 fr_command_tar_uncompress (FRCommand *comm)
 {
 	FRCommandTar *c_tar = FR_COMMAND_TAR (comm);
-	gchar *new_name;
-	gint l;
+	gchar        *new_name;
+	gint          l;
 
 	if (c_tar->uncomp_filename != NULL) {
 		g_free (c_tar->uncomp_filename);
@@ -488,6 +536,7 @@ fr_command_tar_uncompress (FRCommand *comm)
 		if (path_is_file (comm->filename)) {
 			fr_process_begin_command (comm->process, "lzop");
 			fr_process_add_arg (comm->process, "-dfU");
+			fr_process_add_arg (comm->process, "--no-stdin");
 			fr_process_add_arg (comm->process, comm->e_filename);
 			fr_process_end_command (comm->process);
 		}
@@ -510,14 +559,13 @@ fr_command_tar_uncompress (FRCommand *comm)
 static void 
 fr_command_tar_class_init (FRCommandTarClass *class)
 {
+        GObjectClass   *gobject_class = G_OBJECT_CLASS (class);
         FRCommandClass *afc;
-        GtkObjectClass *object_class;
 
-        parent_class = gtk_type_class (FR_COMMAND_TYPE);
-        object_class = (GtkObjectClass*) class;
+        parent_class = g_type_class_peek_parent (class);
 	afc = (FRCommandClass*) class;
 
-        object_class->destroy = fr_command_tar_destroy;
+	gobject_class->finalize = fr_command_tar_finalize;
 
         afc->list         = fr_command_tar_list;
 	afc->add          = fr_command_tar_add;
@@ -537,16 +585,18 @@ fr_command_tar_init (FRCommand *comm)
 	comm->propExtractCanAvoidOverwrite = FALSE;
 	comm->propExtractCanSkipOlder      = FALSE;
 	comm->propExtractCanJunkPaths      = FALSE;
+	comm->propPassword                 = FALSE;
+	comm->propTest                     = FALSE;
 }
 
 
 static void 
-fr_command_tar_destroy (GtkObject *object)
+fr_command_tar_finalize (GObject *object)
 {
 	FRCommandTar *comm_tar;
 
         g_return_if_fail (object != NULL);
-        g_return_if_fail (IS_FR_COMMAND_TAR (object));
+        g_return_if_fail (FR_IS_COMMAND_TAR (object));
 
 	comm_tar = FR_COMMAND_TAR (object);
 
@@ -555,48 +605,49 @@ fr_command_tar_destroy (GtkObject *object)
 		comm_tar->uncomp_filename = NULL;
 	}
 
-	 /* Chain up */
-        if (GTK_OBJECT_CLASS (parent_class)->destroy)
-                (* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
+	/* Chain up */
+        if (G_OBJECT_CLASS (parent_class)->finalize)
+		G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
 
-GtkType
+GType
 fr_command_tar_get_type ()
 {
-        static guint fr_command_tar_type = 0;
+        static guint type = 0;
 
-        if (!fr_command_tar_type) {
-                GtkTypeInfo fr_command_tar_info = {
-                        "FRCommandTar",
-                        sizeof (FRCommandTar),
-                        sizeof (FRCommandTarClass),
-                        (GtkClassInitFunc) fr_command_tar_class_init,
-                        (GtkObjectInitFunc) fr_command_tar_init,
-                        /* reserved_1 */ NULL,
-                        /* reserved_2 */ NULL,
-                        (GtkClassInitFunc) NULL,
-                };
-                fr_command_tar_type = gtk_type_unique (fr_command_get_type(), 
-						       &fr_command_tar_info);
+        if (! type) {
+                GTypeInfo type_info = {
+			sizeof (FRCommandTarClass),
+			NULL,
+			NULL,
+			(GClassInitFunc) fr_command_tar_class_init,
+			NULL,
+			NULL,
+			sizeof (FRCommandTar),
+			0,
+			(GInstanceInitFunc) fr_command_tar_init
+		};
+
+		type = g_type_register_static (FR_TYPE_COMMAND,
+					       "FRCommandTar",
+					       &type_info,
+					       0);
         }
 
-        return fr_command_tar_type;
+        return type;
 }
 
 
 FRCommand *
-fr_command_tar_new (FRProcess *process,
-		    char *filename,
-		    FRCompressProgram prog)
+fr_command_tar_new (FRProcess         *process,
+		    const char        *filename,
+		    FRCompressProgram  prog)
 {
 	FRCommand *comm;
 
-	comm = FR_COMMAND (gtk_type_new (fr_command_tar_get_type ()));
+	comm = FR_COMMAND (g_object_new (FR_TYPE_COMMAND_TAR, NULL));
 	fr_command_construct (comm, process, filename);
-	fr_process_set_proc_line_func (FR_COMMAND (comm)->process, 
-				       process_line,
-				       comm);
 	FR_COMMAND_TAR (comm)->compress_prog = prog;
 	FR_COMMAND_TAR (comm)->uncomp_filename = NULL;
 
